#!/bin/python3

# Copyright (c) 2022 Humanitarian OpenStreetMap Team
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     OSM-Fieldwork is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with OSM-Fieldwork.  If not, see <https:#www.gnu.org/licenses/>.
#

import argparse
import logging
import os
import sys
from datetime import datetime
from pathlib import Path
from sys import argv

from codetiming import Timer

from osm_fieldwork.json2osm import JsonDump
from osm_fieldwork.OdkCentral import OdkAppUser, OdkCentral, OdkForm, OdkProject

# Set log level
log_level = os.getenv("LOG_LEVEL", default="INFO")
logging.getLogger("urllib3").setLevel(log_level)

log = logging.getLogger(__name__)


class OdkClient(object):
    def __init__(
        self,
        url: str = None,
        user: str = None,
        passwd: str = None,
    ):
        """A Class for higher-level client side access to ODK Central.

        Args:
            url (str): The URL of the ODK Central
            user (str): The user's account name on ODK Central
            passwd (str):  The user's account password on ODK Central

        Returns:
            (OdkClient: An instance of this object
        """
        self.url = url
        self.user = user
        self.passwd = passwd
        self.cache = dict()

    # def readCache(self, cache=".odkcentral"):
    #     """FIXME: unimplemented"""
    #     file = open(cache, "rb")
    #     data = file.readline()
    #     print(json.load(data))
    #     file.close()

    # def writeCache(self, cache=".odkcentral", data=None):
    #     """FIXME: unimplemented"""
    #     if args.cache:
    #         try:
    #             file = open(cache, "xb")
    #         except FileExistsError:
    #             file = open(cache, "wb")
    #         file.write("projects\n")
    #         file.write(json.dump(data))
    #         file.close()
    #     logging.info("Wrote config file %s" % filespec)


def main():
    """This main function lets this class be run standalone by a bash script."""
    parser = argparse.ArgumentParser(description="command line client for ODK Central")
    parser.add_argument("-v", "--verbose", action="store_true", help="verbose output")
    # This is for server requests
    parser.add_argument("-s", "--server", choices=["projects", "users", "delete"], help="project operations")
    # This is for project specific requests
    parser.add_argument(
        "-p",
        "--project",
        choices=["forms", "app-users", "assignments", "delete", "submissions"],
        help="project operations",
    )
    parser.add_argument("-i", "--id", type=int, help="Project ID nunmber")
    parser.add_argument("-f", "--form", help="XForm name")
    parser.add_argument("-u", "--uuid", help="Submission UUID, needed to download the data")
    # This is for form specific requests
    parser.add_argument(
        "-x",
        "--xform",
        choices=[
            "attachments",
            "csv",
            "json",
            "submissions",
            "upload",
            "download",
            "create",
            "assignments",
            "delete",
            "publish",
        ],
        help="XForm ID for operations with data files",
    )
    parser.add_argument(
        "-a",
        "--appuser",
        choices=["create", "delete", "update", "qrcode", "access"],
        help="App-User operations",
    )
    parser.add_argument("-d", "--data", help="Data files for upload or download")
    parser.add_argument("-t", "--timestamp", help="Timestamp for submissions")
    parser.add_argument("-b", "--bulk", choices=["qrcodes", "update"], help="Bulk operations")

    # logging.basicConfig(
    #     level=log_level,
    #     format=(
    #         "%(asctime)s.%(msecs)03d [%(levelname)s]"
    #         "%(name)s | %(funcName)s:%(lineno)d | %(message)s"
    #     ),
    #     datefmt="%y-%m-%d %H:%M:%S",
    #     stream=sys.stdout,
    # )

    # Caching isn't implemented yet. That's for fancier queries that require multiple
    # requests to the ODK server. Caching allows for data like names for IDs to
    # be more user friendly.
    # parser.add_argument('-c', '--cache', action="store_true", help = 'cache data from ODK Central')
    # For now read these from the $HOME/.odkcentral config file
    # parser.add_argument('-u', '--user', help = 'ODK Central username (usually email)')
    # parser.add_argument('-pw', '--password', help = 'ODK Central password')
    # parser.add_argument("-d", "--download", choices=['xml', 'xlsx', 'attach', 'submit', 'zip'], help="Download files from ODK Central")
    # parser.add_argument("-u", "--upload", choices=['xml', 'xlsx', 'attach', 'submit', 'zip'], help="Upload files to ODK Central")

    # Any files we want to use are specified on the command line with an argument.
    # Multiple files are stored in a list.
    args, unknown = parser.parse_known_args()

    if len(argv) <= 1:
        parser.print_help()
        quit()

    # Get any files for upload or download
    files = list()
    if unknown is not None:
        files = unknown

    # if verbose, dump to the terminal.
    if args.verbose is not None:
        log.setLevel(logging.DEBUG)
        ch = logging.StreamHandler(sys.stdout)
        ch.setLevel(logging.DEBUG)
        formatter = logging.Formatter("%(threadName)10s - %(name)s - %(levelname)s - %(message)s")
        ch.setFormatter(formatter)
        log.addHandler(ch)

    timer = Timer(text="odk_client() took {seconds:.0f}s")
    timer.start()
    # Commands to the ODK Central server, which gets data that applies
    # to all projects on the server.
    if args.server:
        central = OdkCentral()
        # central.authenticate()
        if args.server == "projects":
            projects = central.listProjects()
            if not projects:
                projects = list()
            elif "message" in projects:
                log.error(f"{projects['message']}, {projects['code']} ")
                quit()

            print("There are %d projects on this ODK Central server" % len(projects))
            ordered = sorted(projects, key=lambda item: item.get("id"))
            for project in ordered:
                print("\t%s: %s" % (project["id"], project["name"]))
        elif args.server == "users":
            users = central.listUsers()
            logging.info("There are %d users on this ODK Central server" % len(users))
            ordered = sorted(users, key=lambda item: item.get("id"))
            for user in ordered:
                print("%s: %s (%s)" % (user["id"], user["displayName"], user["email"]))
        elif args.server == "delete":
            central.deleteProject(args.id)

        # Commands to get data about a specific project on an ODK Central server.
    elif args.project:
        project = OdkProject()
        # project.authenticate()
        if not args.id:
            print('Need to specify a project ID using "--id"!')
            print('You can use "odk_client.-py --server projects" to list all the projects!')
            # parser.print_help()
            quit()
        if args.project == "forms":
            forms = project.listForms(args.id)
            if type(forms) == dict:
                log.error(f"{forms['message']}, {forms['code']} ")
                quit()
            if type(forms) != list:
                log.error(forms["message"])
                quit()
            ordered = sorted(forms, key=lambda item: item.get("xmlFormId"))
            for form in ordered:
                print("\t%r: %r" % (form["xmlFormId"], form["name"]))

        elif args.project == "submissions":
            submissions = project.getAllSubmissions(args.id)
            jsonin = JsonDump()
            now = datetime.now().strftime("%Y-%m-%dT%TZ")
            outfile = f"{args.project}_{now}"
            jsonin.createOSM(f"{outfile}.osm")
            # jsonin.createGeoJson(f"{outfile}.geojson")
            data = jsonin.parse(data=submissions)
            log.debug(f"There are a total of {len(submissions)} submissions")
            for entry in data:
                feature = jsonin.createEntry(entry)
                # Sometimes bad entries, usually from debugging XForm design, sneak in
                if len(feature) == 0:
                    continue
                if "lat" not in feature["attrs"]:
                    if "geometry" in feature["tags"]:
                        if type(feature["tags"]["geometry"]) == str:
                            coords = list(feature["tags"]["geometry"])
                            # del feature['tags']['geometry']
                    elif "coordinates" in feature["tags"]:
                        coords = feature["tags"]["coordinates"]
                        feature["attrs"] = {"lat": coords[1], "lon": coords[0]}
                    else:
                        log.warning("Bad record! %r" % feature)
                        continue
                jsonin.writeOSM(feature)
                log.info(f"Wrote {outfile}.osm")
                # This GeoJson file has all the data values
                # jsonin.writeGeoJson(feature)
                # log.info(f"Wrote {outfile}.geojson")

        elif args.project == "app-users":
            users = project.listAppUsers(args.id)
            logging.info("There are %d app users on this ODK Central server" % len(users))
            ordered = sorted(users, key=lambda item: item.get("id"))
            for user in ordered:
                print("\t%r: %s (%s)" % (user["id"], user["displayName"], user["token"]))
        elif args.project == "delete":
            tmp = files[0].split("-")
            if len(tmp) > 1:
                for id in range(int(tmp[0]), int(tmp[1])):
                    project.deleteProject(id)
            else:
                project.deleteProject(tmp[0])

            # logging.info("There are %d app users on this ODK Central server" %)
            if args.project == "assignments":
                assign = project.listAssignments(args.id)
                logging.info("There are %d assignments  on this ODK Central server" % len(assign))
            ordered = sorted(assign, key=lambda item: item.get("id"))
            for role in ordered:
                print("\t%r" % role)

    elif args.xform:
        # This downloads files from the ODK server
        if not args.id:
            print('Need to specify a project ID using "--id" and an XForm id using "--"!')
            quit()
        if not args.form:
            print('Need to specify a XForm id using "--form"!')
            quit()

        form = OdkForm()
        # form.authenticate()
        # Note that uploading and downloading is only for the attachments, usually
        # a CSV or GeoJson file used by the Form as an external data source for
        # survey questions
        if args.xform == "upload":
            for file in files:
                logging.info("Uploading file %r for XForm %s" % (file, args.form))
                result = form.uploadMedia(args.id, args.form, file)
        elif args.xform == "download":
            logging.info("Downloading files %r for XForm %s" % (files, args.form))
            for file in files:
                logging.info("Downloading %r for XForm %s" % (file, args.form))
                data = form.getMedia(args.id, args.form, file)
                try:
                    file = open(file, "xb")
                except FileExistsError:
                    file = open(file, "wb")
                    file.write(data)
                    file.close()
        elif args.xform == "assignments":
            assign = form.listAssignments(args.id, args.form)
            if type(assign) == dict:
                log.error(f"{assign['message']}, {assign['code']} ")
                quit()
            logging.info("There are %d assignments  on this ODK Central server" % len(assign))
            # ordered = sorted(assign, key=lambda item: item.get('id'))
            for role in assign:
                print("\t%r" % role)

        elif args.xform == "submissions":
            submissions = form.listSubmissions(args.id, args.form)
            if not submissions:
                submissions = list()
            elif "message" in submissions:
                log.error(f"{submissions['message']}, {submissions['code']} ")
                quit()

            logging.info("There are %d submissions for XForm %s" % (len(submissions), args.form))
            for file in submissions:
                # form.submissions.append(file)
                print("%s: %s" % (file["meta"]["instanceID"], file["end"]))

        elif args.xform == "csv":
            submissions = form.getSubmissions(args.id, args.form, True, False)
            if type(submissions) == dict:
                log.error(f"{submissions['message']}, {submissions['code']} ")
            logging.info("There are %d submissions for XForm %s" % (len(submissions), args.form))
            for file in submissions:
                form.submissions.append(file)
                print("%s: %s" % (file["meta"]["instanceID"], file["end"]))

        elif args.xform == "json":
            submissions = form.getSubmissions(args.id, args.form, False, True, True)
            if type(submissions) == dict:
                log.error(f"{submissions['message']}, {submissions['code']} ")
                quit()
            else:
                if submissions is None:
                    submissions = list()
            logging.info("There are %d submissions for XForm %s" % (len(submissions), args.form))
            for file in submissions:
                form.submissions.append(file)
                # print("%s: %s" % (file["instanceId"], file["createdAt"]))

        elif args.xform == "attachments":
            attachments = form.listMedia(args.id, args.form)
            if type(attachments) == dict:
                log.error(f"{attachments['message']}, {attachments['code']} ")
                quit()
            logging.info("There are %d attachments for XForm %s" % (len(attachments), args.form))
            for file in attachments:
                print("\t%s exists ? %s" % (file["name"], file["exists"]))

        elif args.xform == "create":
            for file in files:
                path = Path(file)
                if path.suffix == ".xml":
                    logging.info("Creating XForm from %s" % file)
                    result = form.createForm(args.id, file, args.form)
                elif path.suffix == ".csv":
                    logging.info("Uploading media file %r for XForm %s" % (file, file))
                    result = form.uploadMedia(args.id, args.form, file)
                    result = form.publishForm(args.id, args.form)

        elif args.xform == "delete":
            logging.info("Deleting XForm from %s" % args.form)
            result = form.deleteForm(args.id, args.form)

        elif args.xform == "publish":
            logging.info("Publishing XForm from %s" % args.form)
            result = form.publishForm(args.id, args.form)

    elif args.appuser:
        # This handles app-users
        print("App User ops %s" % args.appuser)
        if not args.id:
            print('Need to specify a project ID using "--id" and an XForm id using "--"!')
            quit()
            # if not args.form:
            #     print("Need to specify a XForm id using \"--form\"!")
            #     quit()

        role = 2  # seems to be the default value
        user = OdkAppUser()
        if args.appuser == "create":
            for appuser in files:
                result = user.create(args.id, appuser)
        elif args.appuser == "delete":
            tmp = files[0].split("-")
            if len(tmp) > 1:
                for id in range(int(tmp[0]), int(tmp[1])):
                    result = user.delete(args.id, id)
            else:
                result = user.delete(args.id, tmp[0])
        elif args.appuser == "update":
            for appuser in files:
                result = user.updateRole(args.id, args.form, role, appuser)
        elif args.appuser == "qrcode":
            result = user.getQRCode(args.id, args.uuid, args.form)
        elif args.appuser == "access":
            for appuser in files:
                result = user.grantAccess(
                    args.id,
                    role,
                    appuser,
                )
                print(result)

        elif args.bulk:
            central = OdkProject()
            # project = central.getDetails(args.id)
            appuser = OdkAppUser()
            role = 2  # thise seems to be the default value
            if not args.form:
                print('Need to specify a XForm id using "--form"!')
                quit()
            if args.bulk == "qrcodes":
                users = central.listAppUsers(args.id)
                for user in users:
                    # name = "%s-%s" % (project['name'], user['displayName'])
                    name = "%s-%s" % (args.form, user["displayName"])
                    result = appuser.getQRCode(args.id, user["token"], name)
            elif args.bulk == "update":
                users = central.listAppUsers(args.id)
                for user in users:
                    result = appuser.updateRole(args.id, args.form, role, user["id"])
    timer.stop()


if __name__ == "__main__":
    """This is just a hook so this file can be run standlone during development."""
    main()



#!/usr/bin/python3

# Copyright (c) 2022, 2023 Humanitarian OpenStreetMap Team
#
# This file is part of osm-Fieldwork.
#
#     Osm-Fieldwork is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     Underpass is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with osm_fieldwork.  If not, see <https:#www.gnu.org/licenses/>.
#

import argparse
import logging
import os
import sys

import geojson
import pandas as pd
from geojson import Feature, FeatureCollection
from osm_rawdata.config import QueryConfig

# from osm_fieldwork import package_root
# from osm_fieldwork.data_models import data_models_path
from osm_fieldwork.xlsforms import xlsforms_path

# Instantiate logger
log = logging.getLogger(__name__)


class FilterData(object):
    def __init__(
        self,
        filespec: str = None,
        config: QueryConfig = None,
    ):
        """Args:
            filespec (str): The optional data file to read.

        Returns:
            (FilterData): An instance of this object
        """
        self.tags = dict()
        self.qc = config
        if filespec and config:
            self.parse(filespec, config)

    def parse(
        self,
        filespec: str,
        config: QueryConfig,
    ):
        """Read in the XLSForm and extract the data we want.

        Args:
            filespec (str): The filespec to the XLSForm file

        Returns:
            title (str): The title from the XLSForm Setting sheet
            extract (str): The data extract filename from the XLSForm Survey sheet
        """
        if config:
            self.qc = config
        excel_object = pd.ExcelFile(filespec)
        entries = excel_object.parse(sheet_name=[0, 1, 2], index_col=0, usercols=[0, 1, 2])
        entries = pd.read_excel(filespec, sheet_name=[0, 1, 2])
        title = entries[2]["form_title"].to_list()[0]
        extract = ""
        for entry in entries[0]["type"]:
            if str(entry) == "nan":
                continue
            if entry[:20] == "select_one_from_file":
                extract = entry[21:]
                log.info(f'Got data extract filename: "{extract}", title: "{title}"')
            else:
                extract = "none"
        total = len(entries[1]["list_name"])
        index = 1
        while index < total:
            key = entries[1]["list_name"][index]
            if key == "model" or str(key) == "nan":
                index += 1
                continue
            value = entries[1]["name"][index]
            if value == "<text>" or str(value) == "null":
                index += 1
                continue
            if key not in self.tags:
                self.tags[key] = list()
            self.tags[key].append(value)
            index += 1

        # The yaml config file for the query has a list of columns
        # to keep in addition to this default set. These wind up
        # in the SELECT
        keep = (
            "name",
            "name:en",
            "id",
            "operator",
            "addr:street",
            "addr:housenumber",
            "osm_id",
            "title",
            "tags",
            "label",
            "landuse",
            "opening_hours",
            "tourism",
        )
        self.keep = list(keep)
        if "keep" in config.config["keep"]:
            self.keep.extend(config.config["keep"])

        return title, extract

    def cleanData(
        self,
        data,
    ):
        """Filter out any data not in the data_model.

        Args:
            data (bytes): The input data or filespec to the input data file

        Returns:
            (FeatureCollection): The modifed data

        """
        log.debug("Cleaning data...")
        if type(data) == str:
            outfile = open(f"new-{data}", "x")
            infile = open(tmpfile, "r")
            indata = geojson.load(infile)
        elif type(data) == bytes:
            indata = eval(data.decode())
        else:
            indata = data
        # these just create noise in the log file
        ignore = (
            "timestamp",
            "version",
            "changeset",
        )
        keep = ("osm_id", "id", "version")
        collection = list()
        for feature in indata["features"]:
            # log.debug(f"FIXME0: {feature}")
            properties = dict()
            for key, value in feature["properties"].items():
                # log.debug(f"{key} = {value}")
                # FIXME: this is a hack!
                if True:
                    if key == "tags":
                        for k, v in value.items():
                            if k[:4] == "name":
                                properties["title"] = value[k]
                                properties["label"] = value[k]
                            else:
                                properties[k] = v
                    else:
                        if key == "osm_id":
                            properties["id"] = value
                            properties["title"] = value
                            properties["label"] = value
                        else:
                            properties[key] = value
                            if key[:4] == "name":
                                properties["title"] = value
                                properties["label"] = value
                else:
                    log.debug(f"FIXME2: {key} = {value}")
                    if key in keep:
                        properties[key] = value
                        continue
                    if key in self.tags:
                        if key == "name" or key == "name:en":
                            properties["title"] = self.tags[key]
                            properties["label"] = self.tags[key]
                        if value in self.tags[key]:
                            properties[key] = value
                        else:
                            if value != "yes":
                                log.warning(f"Value {value} not in the data model!")
                            continue
                    else:
                        if key in ignore:
                            continue
                        log.warning(f"Tag {key} not in the data model!")
                        continue
            if "title" not in properties:
                properties["label"] = properties["id"]
                properties["title"] = properties["id"]
            newfeature = Feature(geometry=feature["geometry"], properties=properties)
            collection.append(newfeature)
        if type(data) == str:
            geojson.dump(FeatureCollection(collection), outfile)
        return FeatureCollection(collection)


def main():
    """This main function lets this class be run standalone by a bash script."""
    parser = argparse.ArgumentParser(description="Convert ODK XML instance file to CSV format")
    parser.add_argument("-v", "--verbose", nargs="?", const="0", help="verbose output")
    parser.add_argument("-i", "--infile", required=True, help="The data extract for ODK Collect")
    parser.add_argument("-x", "--xform", required=True, help="The XForm for ODK Collect")
    parser.add_argument("-o", "--outfile", default="models.yaml", help="The Yaml file of all tags and values")
    args = parser.parse_args()

    # if verbose, dump to the termina
    if args.verbose is not None:
        log.setLevel(logging.DEBUG)
        ch = logging.StreamHandler(sys.stdout)
        ch.setLevel(logging.DEBUG)
        formatter = logging.Formatter("%(threadName)10s - %(name)s - %(levelname)s - %(message)s")
        ch.setFormatter(formatter)
        log.addHandler(ch)

    FilterData()
    xlsforms_path.replace("xlsforms", "data_models")
    models = FilterData()
    data = models.parse(args.xform)
    if args.infile:
        models.cleanData(args.infile)
    else:
        if os.path.exists(args.outfile):
            os.remove(args.outfile)
        outfile = open(args.outfile, "w")
        current = None
        tab = "    "
        outfile.write(
            """
        # Do not edit this file, it is generated by the filter_data.py
        # script from the XForm spreadsheet.
        """
        )
        for key, value in data.items():
            if key != current:
                outfile.write(f"\n{key}:\n")
                for val in value:
                    outfile.write(f"{tab}- {val}\n")


if __name__ == "__main__":
    """This is just a hook so this file can be run standlone during development."""
    main()



#!/usr/bin/python3

# Copyright (c) 2020, 2021, 2022, 2023 Humanitarian OpenStreetMap Team
#
# This file is part of Osm-Fieldwork.
#
#     Osm-Fieldwork is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     Underpass is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with Osm-Fieldwork.  If not, see <https:#www.gnu.org/licenses/>.
#

import argparse
import logging
import sys

import yaml

# Instantiate logger
log = logging.getLogger(__name__)


class YamlFile(object):
    """Config file in YAML format."""

    def __init__(
        self,
        data: str,
    ):
        """This parses a yaml file into a dictionary for easy access.

        Args:
            data (str): The filespec of the YAML file to read

        Returns:
            (YamlFile): An instance of this object
        """
        self.filespec = None
        # if data == str:
        self.filespec = data
        self.file = open(data, "rb").read()
        self.yaml = yaml.load(self.file, Loader=yaml.Loader)
        # else:
        #    self.yaml = yaml.load(str(data), Loader=yaml.Loader)

    def privateData(
        self,
        keyword: str,
    ):
        """See if a keyword is in the private data category.

        Args:
            keyword (str): The keyword to search for

        Returns:
            (bool): Check to see if the keyword is in the private data section
        """
        for value in self.yaml["private"]:
            if keyword.lower() in value:
                return True
        return False

    def ignoreData(
        self,
        keyword: str,
    ):
        """See if a keyword is in the ignore data category.

        Args:
            keyword (str): The keyword to search for

        Returns:
            (bool): Check to see if the keyword is in the ignore data section
        """
        for value in self.yaml["ignore"]:
            if keyword.lower() in value:
                return True
        return False

    def convertData(
        self,
        keyword: str,
    ):
        """See if a keyword is in the convert data category.

        Args:
            keyword (str): The keyword to search for

        Returns:
            (bool): Check to see if the keyword is in the convert data section
        """
        for value in self.yaml["convert"]:
            if keyword.lower() in value:
                return True
        return False

    def dump(self):
        """Dump internal data structures, for debugging purposes only."""
        if self.filespec:
            print("YAML file: %s" % self.filespec)
        for key, values in self.yaml.items():
            print(f"Key is: {key}")
            for v in values:
                if type(v) == dict:
                    for k1, v1 in v.items():
                        if type(v1) == list:
                            for item in v1:
                                for i, j in item.items():
                                    print(f"\t{i} = {j}")
                        else:
                            print(f"\t{k1} = {v1}")
                    print("------------------")
                else:
                    print(f"\t{v}")

    def write(
        self,
        table: list,
    ):
        """Add to the YAML file.

        Args:
              table (list): The name of the database table

        Returns:
            (str): The modified YAML data
        """
        tab = "    "
        yaml = ["select:", f'{tab}"osm_id": id', f"{tab}tags:"]
        for item in where:
            yaml.append(f"{tab}{tab}- {item}")
        yaml.append("from:")
        for item in table:
            yaml.append(f"{tab}- {item}")
        yaml.append("where:")
        yaml.append(f"{tab}tags:")
        notnull = f"{tab}{tab}- " + "{"
        for item in where:
            notnull += f"{item}: NOT NULL, "
        notnull = f"{notnull[:-2]}"
        notnull += "}"
        yaml.append(f"{notnull}")
        return yaml


#
# This script can be run standalone for debugging purposes. It's easier to debug
# this way than using pytest,
#
if __name__ == "__main__":
    """This is just a hook so this file can be run standlone during development."""
    parser = argparse.ArgumentParser(description="Read and parse a YAML file")
    parser.add_argument("-v", "--verbose", action="store_true", help="verbose output")
    parser.add_argument("-i", "--infile", required=True, default="./xforms.yaml", help="The YAML input file")
    args = parser.parse_args()

    # if verbose, dump to the terminal.
    if args.verbose is not None:
        log.setLevel(logging.DEBUG)
        ch = logging.StreamHandler(sys.stdout)
        ch.setLevel(logging.DEBUG)
        formatter = logging.Formatter("%(threadName)10s - %(name)s - %(levelname)s - %(message)s")
        ch.setFormatter(formatter)
        log.addHandler(ch)

    yaml1 = YamlFile(args.infile)
    yaml1.dump()

    table = ("nodes", "ways_poly")
    where = ("building", "amenity", "shop", "tourism")
    # tmp = yaml1.write(table)
    # yaml2 = YamlFile(tmp)
    # yaml2.dump()



#!/usr/bin/python3

#
# Copyright (C) 2020, 2021, 2022, 2023 Humanitarian OpenstreetMap Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

import argparse
import logging
import re
import sys
from datetime import datetime

from osm_fieldwork.convert import Convert
from osm_fieldwork.ODKForm import ODKForm
from osm_fieldwork.ODKInstance import ODKInstance
from osm_fieldwork.osmfile import OsmFile

if __name__ != "__main__":
    print("This is not a loadable python module!")
    exit

parser = argparse.ArgumentParser()
parser.add_argument("-v", "--verbose", action="store_true", help="verbose output")
parser.add_argument("-x", "--xform", required=True, help="input xform file in XML format")
parser.add_argument("-i", "--infile", required=True, help="input data in XML format")
parser.add_argument("-o", "--outdir", help="Output Directory (defaults to $PWD)")
args = parser.parse_args()

# if verbose, dump to the terminal as well as the logfile.
if not args.verbose:
    root = logging.getLogger()
    root.setLevel(logging.DEBUG)

    ch = logging.StreamHandler(sys.stdout)
    ch.setLevel(logging.DEBUG)
    formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
    ch.setFormatter(formatter)
    root.addHandler(ch)

# Get the basename without the suffix
xform = args.xform.replace(".xml", "")
xinst = args.infile.replace(".xml", "")

# This is the output file in OSM format
osmfile = OsmFile(filespec=xinst + ".osm")
osmfile.header()

# Read in the XML config file from ODK Collect
odkform = ODKForm()
odkform.parse(xform + ".xml")

# Read the YAML config file for this XForm
yaml = Convert(xform + ".yaml")
yaml.dump()

# Read the data instance
inst = ODKInstance()
data = inst.parse(args.infile)

out = ""
groups = dict()
tags = dict()
node = dict()
way = dict()
for x in data:
    # print(data[x])
    lat = ""
    lon = ""
    reg = re.compile("group*")
    # There should be only one timestamp we want, namely 'end'
    if odkform.getNodeType(x) == "dateTime":
        dt = data[x][: data[x].find(".")]
        timestamp = datetime.strptime(dt, "%Y-%m-%dT%H:%M:%S")
        groups["timestamp"] = timestamp
        node["timestamp"] = timestamp
        continue
    elif reg.match(x):
        groups[x] = data[x]
        for key, _value in data[x].items():
            if odkform.getNodeType(x, key) == "geotrace":
                ln = "LINESTRING("
                ln += groups[x][key] + ")"
                groups[x][key] = ln
            if odkform.getNodeType(x, key) == "geoshape":
                # print(odkform.getNodeType(x, key))
                py = "POLYGON("
                py += groups[x][key] + ")"
                groups[x][key] = py
            elif odkform.getNodeType(x, key) == "geopoint":
                # print(odkform.getNodeType(x, key))
                tmp = data[x][key].split(" ")
                lat = tmp[0]
                lon = tmp[1]
                node["lat"] = tmp[0]
                node["lon"] = tmp[1]
            elif odkform.getNodeType(x, key) == "list":
                space = data[x][key].find(" ")
                if space <= 0:
                    values = yaml.getKeyword(data[x][key])
                    tags[key] = values
                else:
                    for item in data[x][key].split(" "):
                        if key == "name" or key == "alt_name":
                            tags[key] = data[x][key]
                        else:
                            values = yaml.getKeyword(item)
                            if key in tags:
                                tags[key] = tags[key] + ";" + item
                            else:
                                tags[key] = values
            elif odkform.getNodeType(x, key) == "image":
                pass
            elif odkform.getNodeType(x, key) == "binary":
                pass
            elif odkform.getNodeType(x, key) == "int":
                if data[x][key] is not None:
                    tags[key] = data[x][key]
            elif odkform.getNodeType(x, key) == "string":
                if data[x][key] is not None:
                    tags[key] = data[x][key]
            # else:
            #    tags[x] = key

node["lat"] = lat
node["lon"] = lon
node["tags"] = tags
print("FOO: %r" % node)
out += osmfile.createNode(node)

# We're done
osmfile.write(out)
osmfile.footer()

print("Output file: %s" % xinst + ".osm")
# print(groups)
